var
    expect = require('chai').expect,
    sinon = require('sinon'),
    lodown = require('../index'),
    customers = require('./fixtures/customers.json');

describe('lodown', function() {
    describe('identity', function() {
        it('should return the exact value provided', function() {
            expect(lodown.identity(100)).to.equal(100);
            expect(lodown.identity('Hello World')).to.equal('Hello World');
            expect(lodown.identity(true)).to.equal(true);
            expect(lodown.identity(null)).to.equal(null);

        });
    });
    describe('typeOf', function() {
        it('should return a String for the data type provided', function() {
            expect(lodown.typeOf({})).to.equal('object');
            expect(lodown.typeOf([])).to.equal('array');
            expect(lodown.typeOf(1)).to.equal('number');
            expect(lodown.typeOf(true)).to.equal('boolean');
            expect(lodown.typeOf(null)).to.equal('null');
            expect(lodown.typeOf("Hello World")).to.equal('string');
            expect(lodown.typeOf(new Date())).to.equal('date');
            expect(lodown.typeOf(function() {})).to.equal('function');
            expect(lodown.typeOf(undefined)).to.equal('undefined');
        });
    });    
    describe('first', function() {
        it('should return an Array of FIRST occurence in Array', function() {
            expect(lodown.first(["a","b","c"], 1)).to.equal("a");
            expect(lodown.first(["Tom","Bob","Jerry"],-1)).to.equal([]);
            
        });
    });
    describe('last', function() {
        it('should return an Array of LAST occurence in Array', function() {
            expect(lodown.last([1,2,3,2,4])).to.equal([4]);
            expect(lodown.last(["Tom","Bob","Jerry"], 2)).to.equal(["Bob","Jerry"]);
        });
    });
    describe('each', function() {
        it('should iterate an Array, applying action to each element, index of the element, and the collection', function() {
            var action = sinon.spy();
            lodown.each(customers, action);
            expect(action.callCount).to.equal(customers.length);
            customers.forEach(function(customer, index) {
                expect(action.calledWith(customer, index, customers)).to.be.true;
            });
        });
        it('should iterate an Object, applying action for each value, key of value, and Object', function() {
            var action = sinon.spy();
            var customer = customers[0];
            lodown.each(customer, action);
            expect(action.callCount).to.equal(Object.keys(customer).length);
            for (var key in customer) {
                expect(action.calledWith(customer[key], key, customer)).to.be.true;
            }
        });
    });
    describe('contains', function() {
        it('should return true at first occurance of value in array', function() {
            expect(lodown.contains([1, 2, 3], 2)).to.equal(true);
        });
        it('should return false if value not in array', function() {
            expect(lodown.contains([1, 2, 3], 4)).to.equal(false);
        });
    });
    describe('indexOf', function() {
        var data = ["a","b","c","d", "a"];
        it('Should return the correct index of element in array', function() {
            expect(lodown.indexOf(data, "c")).to.equal(2);
            expect(lodown.indexOf(data, "z")).to.equal(-1);
            expect(lodown.indexOf(data, "a")).to.equal(0);
            expect(lodown.indexOf(data, ["a","b","c","d","a"]));
        });
    });
    describe('reject', function() {
        var data = [1,2,3,4,5];
        it('Should return an array which each element results in a falsey value', function() {
            expect(lodown.reject(data, function(e){return e % 2 == 0})).to.equal([ 1, 3, 5 ]);
        });
    });
});